.thumb { background-color:#F8FCFF;border:1px solid #fff;margin-bottom:0.5em }
.tright { border-width:0.5em 0 0.8em 1.4em;margin:0.5em 0 0.8em 1.4em;clear:right;float:right }
.thumbinner {
background-color:#F9F9F9;
border:1px solid #CCCCCC;
font-size:94%;
overflow:hidden;
padding:3px !important;
text-align:center;
}
.thumbimage { border:1px solid #CCCCCC }

h1, h2, h3, h4, h5, h6 {
  padding-bottom:0.17em;
  padding-top:0.5em;
  color:#000;
}
h2, h1 { border-bottom:1px solid #AAAAAA; }
.editsection {
font-size:76%;
font-weight:normal;
}
.editsection {
float:right;
margin-left:5px;
}

pre {
background-color:#F9F9F9;
border:1px dashed #2F6FAB;
color:black;
line-height:1.1em;
padding:1em;
}
